//
//  DRMovie.h
//  DeepRecorder
//
//  Created by mmivi on 15.04.09.
//  Copyright 2009 DeepIT. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <QuickTime/QuickTime.h>
#import "DREncodingPreset.h"
#import "DREncodedSoundPacket.h"
#import "DREncodedMovieFrame.h"

@interface DRMovie : NSObject {
	DREncodingPreset * _preset;
	NSString * _moviePath;
	
	Movie _movie;
	DataHandler _movieDataHandler;
	
	Track _videoTrack;
	Media _videoTrackMedia;
	BOOL _keepRunningVideo, _stopNowVideo, _pauseVideo, _finalizeVideo;
	id _videoDataSource;
	
	Track _audioTrack;
	Media _audioTrackMedia;
	SoundDescriptionHandle _soundDescription;
	AudioStreamBasicDescription _destinationASBD;
	BOOL _keepRunningAudio, _stopNowAudio, _pauseAudio, _finalizeAudio;
	id _audioDataSource;
	
	Track _systemAudioTrack;
	Media _systemAudioTrackMedia;
	SoundDescriptionHandle _systemSoundDescription;
	AudioStreamBasicDescription _systemDestinationASBD;
	BOOL _systemKeepRunningAudio, _systemStopNowAudio, _systemPauseAudio, _systemFinalizeAudio;
	id _systemAudioDataSource;
	
	CFAbsoluteTime _systemAudioOffset, _audioOffset, _videoOffset;
	
	id _delegate;
}

- (id) initWithPreset: (DREncodingPreset *)preset;
- (int)createMovieAtPath: (NSString *)path;
- (Movie)movie;

- (Track)videoTrack;
- (int)createVideoTrackWithInitialSize: (NSSize)initSize;
- (int)createVideoTrackWithInitialSize: (NSSize)initSize :(TimeScale)timeScale;
- (void)startVideoTrackWriting;
- (void)pauseVideoTrackWriting: (BOOL)flag;
- (void)stopVideoTrackWriting;
- (void)finalizeVideo;

- (int)createAudioTrackWithAudioDescriptionHandle: (SoundDescriptionHandle)descr;
- (void)startAudioTrackWriting;
- (void)pauseAudioTrackWriting: (BOOL)flag;
- (void)stopAudioTrackWriting;
- (void)finalizeAudio;

- (int)createSystemAudioTrackWithAudioDescriptionHandle: (SoundDescriptionHandle)descr;
- (void)startSystemAudioTrackWriting;
- (void)pauseSystemAudioTrackWriting: (BOOL)flag;
- (void)stopSystemAudioTrackWriting;
- (void)finalizeSystemAudio;

- (void)updateMovie;
- (int)closeMovie;

- (void)setVideoDataSource: (id)object;
- (void)setAudioDataSource: (id)object;
- (void)setSystemAudioDataSource: (id)object;

- (void)setDelegate: (id)object;

- (void)setAudioTrackOffset: (CFAbsoluteTime)offset;
- (void)setVideoTrackOffset: (CFAbsoluteTime)offset;
- (void)setSystemAudioTrackOffset: (CFAbsoluteTime)offset;

@end

@interface NSObject (DRMovieDelegateInfromalProtocol)

- (void)DRMovieDidStopWritingAudio: (DRMovie *)theMovie;
- (void)DRMovieDidStopWritingVideo: (DRMovie *)theMovie;
- (void)DRMovieDidStopWritingSystemAudio: (DRMovie *)theMovie;

@end
