//
//  DRSoundEncoder.h
//  DeepRecorder
//
//  Created by mmivi on 30.03.09.
//  Copyright 2009 DeepIT. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CoreAudio/CoreAudio.h>
#import <AudioToolbox/AudioToolbox.h>
#import <QuickTime/QuickTime.h>
#import "DREncodingPreset.h"
#import "DRSoundPacket.h"
#import "DREncodedSoundPacket.h"

#define DR_AUENC_NUMFRAMES_KEY @"NumFrames"

@interface NSObject (DRSoundEncoderDataDestinationInfromalProtocol)

- (void)addObject: (id)object;
- (BOOL)canAcceptObject;

@end

@interface DRSoundEncoder : NSObject {
	DREncodingPreset * _preset;
	id _delegate;
	id _dataSource;
	id _dataDestination;
	
	AudioConverterRef _converter;
	AudioStreamBasicDescription _sourceASBD, _destinationASBD;
	Boolean bFormatRequiresPacketDescriptions;
	AudioChannelLayout *dstLayout;
	UInt32 dstLayoutSize;
	UInt32 uiRenderQuality;
	
	CFArrayRef aCodecSpecificSettings;
	void * pMagicCookie;
	UInt32 uiMagicCookieSize;
	DRSoundFormat _sourceFormat;
	
	BOOL _keepRunning, _stopNow, _isPaused, _finalize;
	
	DRSoundPacket * _currentPacket;
	int _currentPacketOffset;
}

- (id) initWithPreset: (DREncodingPreset *)preset sourceSoundFormat: (DRSoundFormat)fmt;
+ (AudioStreamBasicDescription)LPCMASBDForFormat: (DRSoundFormat)fmt;

- (void)getPackets: (void **)buffer count: (int *)num;
- (void)getSoundDescription: (SoundDescriptionHandle *)hndl;
- (void)setSourceFormat: (DRSoundFormat)fmt;
- (DRSoundFormat)sourceFormat;
- (void)start;
- (void)stop;
- (BOOL)isPaused;
- (void)pause: (BOOL)flag;

- (void)setDataSource: (id)object;
- (void)setDataDestination: (id)object;
- (void)setDelegate: (id)object;

@end

@interface NSObject (DRSoundEncoderDelegateInfromalProtocol)

- (void)DRSoundEncoderDidStop: (DRSoundEncoder *)soundEncoder;
- (void)DRSoundEncoder: (DRSoundEncoder *)soundEncoder willEncodeFrames: (int)num;

@end
