//
//  DRSoundInputRecorder.h
//  DeepRecorder
//
//  Created by mmivi on 25.03.09.
//  Copyright 2009 DeepIT. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CoreAudio/CoreAudio.h>
#import "DRSoundPacket.h"
#import <OpenAL/OpenAL.h>

#define DR_SIR_PAUSE_KEY @"PauseStateKey"
#define DR_SIR_DEFAULT_SAFETY_SLICE 1024

extern const ALCenum kSoundInputRecorderDefultFormat;
extern const float kSoundInputRecorderDefultSampleRate;
extern const int kSoundInputRecorderDefultSliceSize;

@interface NSObject (DRSoundInputRecorderDataDestinationInformalProtocol)

- (void)addObject: (DRSoundPacket *)object;
- (BOOL)canAcceptObject;

@end

@interface DRSoundInputRecorder : NSObject {
	NSString * _deviceName;
	ALCdevice * _inputDevice;
	float _sampleRate;
	int _framesInSlice;
	int _safetyFramesInSilce;
	ALCenum _alFormat;
	
	int _totalFramesRead;
	DRSoundFormat _drFormat;
	
	NSLock * _deviceLock;
	
	id _dataDestination;
	id _delegate;
	
	BOOL _isPaused;
	BOOL _keepRunning;
	BOOL _stopNow;
	
	CFAbsoluteTime _realStartTime, _startOffsetTime;
	
	BOOL _suspend;
}

+ (NSArray *)availableDevicesIsInput: (Boolean)isInput;
+ (NSArray *)availableDevices;
+ (AudioDeviceID)deviceIdForDeviceName: (NSString *)name isInput: (Boolean)isInput;

- (id) initWithDevice: (NSString *)deviceName oalFormat: (ALenum)format frequency: (float)freq sliceSize: (int)size;
- (id) initWithDevice: (NSString *)deviceName;
- (DRSoundFormat)format;

- (BOOL)isSuspended;
- (void)setSuspended:(BOOL)flag;

- (void)setSafetySlice: (int)value;
- (CFAbsoluteTime)startOffsetTime;
- (void)startAtTime: (CFAbsoluteTime)theTime;

- (void)start;
- (void)stop;
- (BOOL)isPaused;
- (void)pause: (BOOL)flag;

- (id)delegate;
- (void)setDelegate: (id)object;
- (id)dataDestination;
- (void)setDataDestination: (id)object;

@end

@interface NSObject (DRSoundInputRecorderDelegateInformalProtocol)

- (void)DRSoundInputRecorderDidStart: (DRSoundInputRecorder *)soundRecorder;
- (void)DRSoundInputRecorderDidStop: (DRSoundInputRecorder *)soundRecorder;
- (void)DRSoundInputRecorder: (DRSoundInputRecorder *)soundRecorder didChangePauseState: (BOOL)newState;
- (void)DRSoundInputRecorderShouldStart: (DRSoundInputRecorder *)soundRecorder;
- (void)DRSoundInputRecorderWillStart:(DRSoundInputRecorder *)soundRecorder;

@end
